@@ -83,7 +83,7 @@ def cal_kl(m, v, use_ne='auto'):
 
 
 def cluster(model, data, device, max_edges, max_node, is_combined,
-            logger, n_sample, out, contig_dict, binned_length, num_process, minfasta, recluster, random_seed):
+            logger, n_sample, out, contig_dict, binned_length, num_process, minfasta, recluster, random_seed, orf_finder = 'prodigal'):
     """
     Cluster contigs into bins
     max_edges: max edges of one contig considered in binning
@@ -94,6 +94,7 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
     from sklearn.cluster import KMeans
     from scipy import sparse
     import torch
+    import pandas as pd
     import numpy as np
     train_data = data.values
     train_data_input = train_data[:, 0:136] if not is_combined else train_data
@@ -142,7 +143,7 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
     embedding_matrix.data[embedding_matrix.data<= threshold] = 0
     embedding_matrix.eliminate_zeros()
     if not is_combined:
-        logger.info('Calculating depth matrix.')
+        logger.debug('Calculating depth matrix.')
         kl_matrix = None
         for k in range(n_sample):
             kl = cal_kl(depth[:,2*k], depth[:, 2*k + 1])
@@ -165,7 +166,7 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
     edges = [(x,y) for x,y in zip(X, Y)]
     edge_weights = V[above_diag]
 
-    logger.info('Edges:{}'.format(len(edges)))
+    logger.debug(f'Number of edges in clustering graph: {len(edges)}')
 
     g = Graph()
     g.add_vertices(np.arange(num_contigs))
@@ -180,13 +181,22 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
     for i, r in enumerate(result):
         contig_labels[r] = i
 
-    output_bin_path = os.path.join(out, 'output_bins')
+    output_bin_path = os.path.join(out, 'output_prerecluster_bins' if recluster else 'output_bins')
     if os.path.exists(output_bin_path):
         shutil.rmtree(output_bin_path)
     os.makedirs(output_bin_path, exist_ok=True)
 
     bin_files = write_bins(namelist, contig_labels, output_bin_path, contig_dict,minfasta=minfasta)
-    if recluster:
+    if not len(bin_files):
+        logger.warning('No bins were created. Please check your input data.')
+        return
+
+    if not recluster:
+        logger.info(f'Number of bins: {len(bin_files)}')
+        bin_files.to_csv(os.path.join(out, 'bins_info.tsv'), index=False, sep='\t')
+
+    else:
+        logger.info(f'Number of bins prior to reclustering: {len(bin_files)}')
         if not is_combined:
             mean_index = [2 * temp for temp in range(n_sample)]
             depth_mean = depth[:, mean_index]
@@ -200,7 +210,7 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
         else:
             embedding_new = embedding
 
-        logger.info('Reclustering.')
+        logger.debug('Reclustering.')
 
         output_recluster_bin_path = os.path.join(out, 'output_recluster_bins')
         if os.path.exists(output_recluster_bin_path):
@@ -211,7 +221,7 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
         with tempfile.TemporaryDirectory() as tdir:
             cfasta = os.path.join(tdir, 'concatenated.fna')
             with open(cfasta, 'wt') as concat_out:
-                for ix,f in enumerate(bin_files):
+                for ix,f in enumerate(bin_files['filename'].values):
                     for h,seq in fasta_iter(f):
                         h = f'bin{ix:06}.{h}'
                         concat_out.write(f'>{h}\n{seq}\n')
@@ -219,9 +229,13 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
                 cfasta,
                 binned_length,
                 num_process,
-                multi_mode=True)
+                multi_mode=True,
+                orf_finder=orf_finder)
 
-        for ix,bin_path in enumerate(bin_files):
+        outputs = []
+        # The code below (iat call) relies on this
+        assert bin_files.columns[0] == 'filename'
+        for ix,bin_path in enumerate(bin_files['filename'].values):
             # if there are no hits, the output will be naturally empty
             seed = seeds.get(f'bin{ix:06}', [])
             num_bin = len(seed)
@@ -242,10 +256,16 @@ def cluster(model, data, device, max_edges, max_node, is_combined,
                     random_state=random_seed)
                 kmeans.fit(re_bin_features, sample_weight=length_weight)
                 labels = kmeans.labels_
-                write_bins(contig_list, labels, os.path.join(out, 'output_recluster_bins'), contig_dict,
+                part = write_bins(contig_list, labels, os.path.join(out, 'output_recluster_bins'), contig_dict,
                            recluster=True, origin_label=int(bin_path.split('.')[-2]),minfasta = minfasta)
+                outputs.append(part)
             else:
-                shutil.copy(bin_path, os.path.join(out, 'output_recluster_bins'))
+                ofname = shutil.copy(bin_path, os.path.join(out, 'output_recluster_bins'))
+                bin_files.iat[ix, 0] = ofname
+                outputs.append(bin_files[ix:ix+1])
 
+        outputs = pd.concat(outputs)
+        logger.info(f'Number of bins after reclustering: {len(outputs)}')
+        outputs.to_csv(os.path.join(out, 'recluster_bins_info.tsv'), index=False, sep='\t')
     logger.info('Binning finish.')
 
