@@ -1,6 +1,6 @@
 import os
 import subprocess
-from atomicwrites import atomic_write
+from .atomicwrite import atomic_write
 
 def calculate_coverage(depth_stream, bam_file, must_link_threshold, edge=75, is_combined=False,
                        contig_threshold=1000, sep=None, contig_threshold_dict=None):
@@ -29,7 +29,7 @@ def calculate_coverage(depth_stream, bam_file, must_link_threshold, edge=75, is_
         for line in lines:
             line_split = line.strip().split('\t')
             length = int(line_split[2]) - int(line_split[1])
-            value = int(line_split[3])
+            value = int(float(line_split[3]))
             lengths.append(length)
             values.append(value)
         depth_value = np.zeros(sum(lengths), dtype=int)
@@ -87,9 +87,8 @@ def generate_cov(bam_file, bam_index, out, threshold,
     sep: separator for multi-sample binning
     """
     import numpy as np
-    logger.info('Processing `{}`'.format(bam_file))
+    logger.debug('Processing `{}`'.format(bam_file))
     bam_name = os.path.split(bam_file)[-1] + '_{}'.format(bam_index)
-    bam_depth = os.path.join(out, '{}_depth.txt'.format(bam_name))
 
     bed_p = subprocess.Popen(
         ['bedtools', 'genomecov',
@@ -101,6 +100,7 @@ def generate_cov(bam_file, bam_index, out, threshold,
     if is_combined:
         contig_cov, must_link_contig_cov = calculate_coverage(bed_p.stdout, bam_file, threshold, is_combined = is_combined, sep = sep, contig_threshold = contig_threshold if sep is None else 1000, contig_threshold_dict =  contig_threshold if sep is not None else None)
         if bed_p.wait() != 0:
+            logger.critical("Running `bedtools genomecov` failed. Please check your input files: SemiBin expects that they are sorted BAM files.")
             raise OSError("Failure in running bedtools")
 
         contig_cov = contig_cov.apply(lambda x: x + 1e-5)
