@@ -139,7 +139,7 @@ class unsupervised_feature_Dataset(Dataset):
 
 
 def train(out, contig_fastas, binned_lengths, logger, datas, data_splits, cannot_links, is_combined=True,
-          batchsize=2048, epoches=20, device=None, num_process = 8, mode = 'single'):
+          batchsize=2048, epoches=20, device=None, num_process = 8, mode = 'single', orf_finder = 'prodigal'):
     """
     Train model from one sample(--mode single) or several samples(--mode several)
     """
@@ -170,9 +170,11 @@ def train(out, contig_fastas, binned_lengths, logger, datas, data_splits, cannot
             seed = cal_num_bins(
                 contig_fastas[data_index],
                 binned_length=binned_lengths[data_index],
-                num_process=num_process)
+                num_process=num_process,
+                output=os.path.join(out, 'sample{}'.format(data_index)),
+                orf_finder=orf_finder)
             if epoch == 0:
-                logger.info('Generate training data of {}:'.format(data_index))
+                logger.debug('Generate training data of {}:'.format(data_index))
 
             # #generate two inputs
             train_input_1 = []
@@ -233,10 +235,10 @@ def train(out, contig_fastas, binned_lengths, logger, datas, data_splits, cannot
                 train_labels.append(1)
 
             if epoch == 0:
-                logger.info(
-                    'Number of must link pair:{}'.format(train_labels.count(1)))
-                logger.info(
-                    'Number of can not link pair:{}'.format(train_labels.count(0)))
+                logger.debug(
+                    'Number of must-link pairs: {}'.format(train_labels.count(1)))
+                logger.debug(
+                    'Number of cannot-link pairs: {}'.format(train_labels.count(0)))
 
             dataset = feature_Dataset(train_input_1, train_input_2,
                                       train_labels)
